from flask.ext.wtf import Form, TextField, BooleanField, TextAreaField, PasswordField
from flask.ext.babel import lazy_gettext
from wtforms.ext.sqlalchemy.orm import model_form
from flask.ext.wtf import Required, Length, validators, EqualTo
from flask.ext.appbuilder.forms import DynamicForm, BS3PasswordFieldWidget

class LoginForm_oid(DynamicForm):
    openid = TextField(lazy_gettext('openid'), validators = [Required()])
    remember_me = BooleanField(lazy_gettext('remember_me'), default = False)

class LoginForm_db(DynamicForm):
    username = TextField(lazy_gettext('User Name'), validators = [Required()])
    password = PasswordField(lazy_gettext('Password'), validators = [Required()])


class ResetPasswordForm(DynamicForm):
    password = PasswordField(lazy_gettext('Password'),
            description=lazy_gettext('Please use a good password policy, this application does not check this for you'),
            validators = [Required()],
            widget=BS3PasswordFieldWidget())
    conf_password = PasswordField(lazy_gettext('Confirm Password'),
            description=lazy_gettext('Please rewrite the password to confirm'),
            validators=[EqualTo('password',message=lazy_gettext('Passwords must match'))],
            widget=BS3PasswordFieldWidget())




class ActionItem():
    name = ""
    text = ""
    func = None

    def __init__(self, name, text, func):
        self.name = name
        self.text = text
        self.func = func

def action(name, text, confirmation=None):
    """
        Use this decorator to expose actions that span more than one
        entity (model, file, etc)

        :param name:
            Action name
        :param text:
            Action text.
        :param confirmation:
            Confirmation text. If not provided, action will be executed
            unconditionally.
    """
    def wrap(f):
        f._action = (name, text, confirmation)
        return f

    return wrap



# -*- coding: utf-8 -*-

from flask import flash
from flask.ext.babel import gettext, ngettext, lazy_gettext
from sqlalchemy.orm import class_mapper, joinedload
from sqlalchemy.exc import IntegrityError
from sqlalchemy import MetaData
from sqlalchemy import func

import sqlalchemy as sa
import sys
from mixins import FileColumn, ImageColumn
from ..filemanager import FileManager, ImageManager


class DataModel():
    obj = None
    
    """ Messages to display on CRUD Events """
    add_row_message = lazy_gettext('Added Row')
    edit_row_message = lazy_gettext('Changed Row')
    delete_row_message = lazy_gettext('Deleted Row')
    delete_integrity_error_message = lazy_gettext('Associated data exists, please delete them first')
    add_integrity_error_message = lazy_gettext('Integrity error, probably unique constraint')
    edit_integrity_error_message = lazy_gettext('Integrity error, probably unique constraint')
    general_error_message = lazy_gettext('General Error')

    def __init__(self, obj):
        self.obj = obj

class SQLAModel(DataModel):
    """
    SQLAModel
    Implements SQLA support methods for views
    """
    session = None
    
    def __init__(self, obj, session = None):
        self.session = session
        DataModel.__init__(self, obj)

    
    def _get_base_query(self, query = None, filters = {}, order_column = '', order_direction = ''):
        for filter_key in filters:
            try:
                rel_model, rel_direction = self._get_related_model(filter_key)
                item = (filters.get(filter_key))
                if rel_direction == 'MANYTOONE':
                    query = query.filter(getattr(self.obj,filter_key) == item)
                elif rel_direction == 'MANYTOMANY':
                    query = query.filter(getattr(self.obj,filter_key).contains(item))
                else:
                    pass
            except:
                if isinstance(self.obj.__mapper__.columns[filter_key].type, sa.types.String):
                    query = query.filter(getattr(self.obj,filter_key).like(filters.get(filter_key) + '%%'))
        if (order_column != ''):
            query = query.order_by(order_column + ' ' + order_direction)
        return query
    
    """
    QUERY
    filters: dict with filters {<col_name>:<value,...}
    order_column
    order_direction: <'asc'|'desc'>
    """
    def query(self, filters = {}, order_column = '', order_direction = ''):
        query = self.obj.query
        query = self._get_base_query(query = query, filters = filters, order_column = order_column, order_direction = order_direction)
        return query.all()


    def query_simple_group(self, group_by = '', filters = {}, order_column = '', order_direction = ''):
        try:
            rel_model, rel_direction = self._get_related_model(group_by)
            query = self.session.query(rel_model,func.count(self.obj.id))
            query = query.join(rel_model, getattr(self.obj,group_by))
            query = self._get_base_query(query = query, filters = filters, order_column = order_column, order_direction = order_direction)
            query = query.group_by(rel_model)
            return query.all()
        except:
            query = self.session.query(rel_model,func.count(self.obj.id))
            query = self._get_base_query(query = query, filters = filters, order_column = order_column, order_direction = order_direction)
            query = query.group_by(group_by)
            return query.all()


    """
    -----------------------------------------
         FUNCTIONS for Testing TYPES
    -----------------------------------------
    """
    def is_image(self, col_name):
        return isinstance(self.obj.__mapper__.columns[col_name].type, ImageColumn)

    def is_file(self, col_name):
        return isinstance(self.obj.__mapper__.columns[col_name].type, FileColumn)

    def is_string(self, col_name):
        return isinstance(self.obj.__mapper__.columns[col_name].type, sa.types.String)

    def is_text(self, col_name):
        return isinstance(self.obj.__mapper__.columns[col_name].type, sa.types.Text)

    def is_integer(self, col_name):
        return isinstance(self.obj.__mapper__.columns[col_name].type, sa.types.Integer)

    def is_boolean(self, col_name):
        return isinstance(self.obj.__mapper__.columns[col_name].type, sa.types.Boolean)

    def is_date(self, col_name):
        return isinstance(self.obj.__mapper__.columns[col_name].type, sa.types.Date)

    def is_datetime(self, col_name):
        return isinstance(self.obj.__mapper__.columns[col_name].type, sa.types.DateTime)

    def is_relation(self, prop):
        return isinstance(prop, sa.orm.properties.RelationshipProperty)

    def is_relation_col(self, col):
        for i in self.get_properties_iterator():
            if self.is_relation(i):
                if (i.key == col):
                    return self.is_relation(i)
        return False

    def is_relation_many_to_one(self, prop):
        return (prop.direction.name == 'MANYTOONE')

    def is_relation_many_to_many(self, prop):
        return (prop.direction.name == 'MANYTOMANY')

    def is_pk(self, col):
        return col.primary_key

    def is_fk(self, col):
        return  col.foreign_keys

    """
    -----------------------------------------
           FUNCTIONS FOR CRUD OPERATIONS
    -----------------------------------------
    """
    def add(self, item):
        try:
            self.session.add(item)
            self.session.commit()
            flash(unicode(self.add_row_message),'success')
            return True
        except IntegrityError as e:
            flash(unicode(self.add_integrity_error_message),'warning')
            self.session.rollback()
            return False
        except:
            flash(unicode(self.general_error_message + ' '  + str(sys.exc_info()[0])),'danger')
            self.session.rollback()
            return False

    def edit(self, item):
        try:
            self.session.merge(item)
            self.session.commit()
            flash(unicode(self.edit_row_message),'success')
            return True
        except IntegrityError as e:
            flash(unicode(self.edit_integrity_error_message),'warning')
            self.session.rollback()
            return False
        except:
            flash(unicode(self.general_error_message + ' '  + str(sys.exc_info()[0])),'danger')
            self.session.rollback()
            return False
                
    
    def delete(self, item):
        try:
            self._delete_files(item)
            self.session.delete(item)
            self.session.commit()
            flash(unicode(self.delete_row_message),'success')
            return True
        except IntegrityError as e:
            flash(unicode(self.delete_integrity_error_message),'warning')
            self.session.rollback()
            return False
        except:
            flash(unicode(self.general_error_message + ' '  + str(sys.exc_info()[0])),'danger')
            self.session.rollback()
            return False
        
        
    """
    FILE HANDLING METHODS
    """
    def _add_files(self, this_request, item):
        fm = FileManager()
        im = ImageManager()
        for file_col in this_request.files:
            if self.is_file(file_col):                
                fm.save_file(this_request.files[file_col],getattr(item, file_col))
        for file_col in this_request.files:
            if self.is_image(file_col):
                im.save_file(this_request.files[file_col], getattr(item,file_col))
                
        
    def _delete_files(self, item):
        for file_col in self.get_file_column_list():
            if self.is_file(file_col):
                if getattr(item,file_col):
                    fm = FileManager()
                    fm.delete_file(getattr(item, file_col))
        for file_col in self.get_image_column_list():
            if self.is_image(file_col):
                if getattr(item,file_col):
                    im = ImageManager()
                    im.delete_file(getattr(item,file_col))

    """
    -----------------------------------------
         FUNCTIONS FOR RELATED MODELS
    -----------------------------------------
    """
    def get_model_relation(self, prop):
        return prop.mapper.class_

    def get_property_col(self, prop):
        return prop.key

    def _get_related_model(self, col_name):
        for i in self.get_properties_iterator():
            if self.is_relation(i):
                if (i.key == col_name):
                    return self.get_model_relation(i), i.direction.name
        return None

    def get_related_obj(self, col_name, value):
        rel_model, rel_direction = self._get_related_model(col_name)
        return rel_model.query.get(value)

    def get_related_fks(self, related_views):
        return [view.datamodel.get_related_fk(self.obj) for view in related_views]

    def get_related_fk(self, model):
        for i in self.get_properties_iterator():
            if self.is_relation(i):
                if model == self.get_model_relation(i):
                    return self.get_property_col(i)

    def get_relation_filters(self, filters = {}):
        return [filter_key for filter_key in filters if self.is_relation_col(filter_key)]
    

    """
    ----------- GET METHODS -------------
    """
    def get_properties_iterator(self):
        return sa.orm.class_mapper(self.obj).iterate_properties

    def get_columns_list(self):
        ret_lst = []
        for prop in self.get_properties_iterator():
            if not self.is_relation(prop):
                if (not self.is_pk(self.get_property_first_col(prop))) and (not self.is_fk(self.get_property_first_col(prop))):
                    ret_lst.append(prop.key)
            else:
                ret_lst.append(prop.key)
        return ret_lst

    def get_file_column_list(self):
        return [i.name for i in self.obj.__mapper__.columns if self.is_file(i.name)]
        
    def get_image_column_list(self):
        return [i.name for i in self.obj.__mapper__.columns if self.is_image(i.name)]
    

    def get_property_first_col(self, prop):
        # support for only one col for pk and fk
        return prop.columns[0]

    def get_col_property(self, colname):
        for prop in self.get_properties_iterator():
            if colname == prop.key:
                return prop

    def get(self, id):
        return self.obj.query.get(id)

    def get_col_byname(self, name):
        return getattr(self.obj, name)
        
    def _get_attr_value(self, item, col):
        if hasattr(getattr(item, col), '__call__'):
            # its a function
            return getattr(item, col)()
        else:
            # its attribute
            return getattr(item, col)

    def get_values_item(self, item, show_columns):
        return [self._get_attr_value(item, col) for col in show_columns]
        
    """
    ----------- GET VALUES -----------------
    """
    def get_values(self, lst, list_columns):
        retlst = []
        for item in lst:
            retdict = {}
            for col in list_columns:
                    retdict[col] = self._get_attr_value(item,col)
            retlst.append(retdict)
        return retlst

    """
    ----------- GET KEYS -------------------
    """
    def get_keys(self, lst):
        pk_name = self.get_pk_name()
        return [getattr(item, pk_name) for item in lst]
        
        
    """
    ----------- GET PK NAME -------------------
    """
    def get_pk_name(self):
        retstr = ""
        for item in list(self.obj.__mapper__.columns):
            if item.primary_key:
                retstr = item.name
                break
        return retstr

    def get_pk_value(self, item):
        for col in list(self.obj.__mapper__.columns):
            if col.primary_key:
                return getattr(item,col.name)

    def printdebug(self):
        for item in list(self.obj.__mapper__.columns):
            print item.name, ' ', item.type, item.primary_key, item.nullable



import os
import shutil
from flask.ext import appbuilder

#projectname = raw_input('Enter you project name: ')
#os.mkdir(projectname)
#path = os.path.dirname(appbuilder.__file__)
#shutil.copytree(str(path).join('/skeleton') , './'.join(projectname))


from app import app, db
from flask.ext.appbuilder.security.models import User, Role


print "--------------------------"
print "     CREATE DB"
print "--------------------------"
db.create_all()
print "--------------------------"
print " CREATE USER and  ROLES"
print "--------------------------"
role_admin = Role()
role_admin.name = app.config['AUTH_ROLE_ADMIN']
role_public = Role()
role_public.name = app.config['AUTH_ROLE_PUBLIC']
user = User()
user.first_name = 'Admin'
user.last_name = ''
user.username = 'admin'
user.password = 'general'
user.active = True
user.role = role_admin

db.session.add(role_admin)
db.session.add(role_public)
db.session.add(user)
db.session.commit()



import re
from flask import Blueprint, render_template, flash, redirect, url_for, request, send_file
from flask.ext.login import login_required
from flask.ext.babel import gettext, ngettext, lazy_gettext
from forms import GeneralModelConverter
from .security.decorators import has_access
from .filemanager import uuid_originalname
from .widgets import FormWidget, ShowWidget, ListWidget, SearchWidget



def expose(url='/', methods=('GET',)):
    """
        Use this decorator to expose views in your view classes.
    """
    def wrap(f):
        if not hasattr(f, '_urls'):
            f._urls = []
        f._urls.append((url, methods))
        return f
    return wrap


class BaseView(object):
    """
    Base View for all Views
    includes permission, app info
    """

    baseapp = None
    blueprint = None
    endpoint = None
    
    route_base = None
    template_folder = 'templates'
    static_folder='static'
    
    base_permissions = None
    redirect_url = '/'

    def __init__(self):
        if not self.base_permissions:
            self.base_permissions = [
                ('can_' + attr_name) 
                for attr_name in dir(self) 
                if hasattr(getattr(self, attr_name),'_urls')
                ]


    def create_blueprint(self, baseapp, 
                        endpoint = None, 
                        static_folder = None):
        """
            Create Flask blueprint.
        """
        # Store BaseApp instance
        self.baseapp = baseapp

        # If endpoint name is not provided, get it from the class name
        if not self.endpoint:
            self.endpoint = self.__class__.__name__.lower()

        self.static_folder = static_folder
        if not static_folder:
        # Create blueprint and register rules
            self.blueprint = Blueprint(self.endpoint, __name__,
                                   url_prefix=self.route_base,
                                   template_folder=self.template_folder)
        else:
            self.blueprint = Blueprint(self.endpoint, __name__,
                                   url_prefix=self.route_base,
                                   template_folder=self.template_folder,
                                   static_folder = static_folder)

        self.register_urls()
        return self.blueprint

    def register_urls(self):
        for attr_name in dir(self):
            attr = getattr(self, attr_name)

            if hasattr(attr, '_urls'):
                for url, methods in attr._urls:
                    self.blueprint.add_url_rule(url,
                                        attr_name,
                                        attr,
                                        methods=methods)
        

    def _prettify_name(self, name):
        return re.sub(r'(?<=.)([A-Z])', r' \1', name)


    def _get_redirect(self):
        if (request.args.get('next')):
            return request.args.get('next')
        else:
            return self.redirect_url

    def _get_group_by_args(self):
        group_by = request.args.get('group_by')
        if not group_by: group_by = ''
        return group_by

    def _get_order_args(self):
        order_column = request.args.get('order_column')
        order_direction = request.args.get('order_direction')
        if not order_column: order_column = ''
        if not order_direction: order_direction = ''
        return order_column, order_direction

    def _get_filter_args(self, filters={}):
        for n in request.args:
            re_match = re.findall('_flt_(.*)', n)
            if re_match:
                filters[re_match[0]] = request.args.get(n)
        return filters


    def _get_dict_from_form(self, form, filters={}):
        for item in form:
            if item.data:
                filters[item.name] = item.data
        return filters



class IndexView(BaseView):
    """
    Index View
    A simple view that implements the index for the site
    """

    route_base = ''

    index_template = 'appbuilder/index.html'

    @expose('/')
    @expose('/index/')
    def index(self):
        return render_template(self.index_template, baseapp = self.baseapp)

"""
----------------------------------
     SIMPLE FORM VIEW
----------------------------------
"""
class SimpleFormView(BaseView):
    """
    Simple Form View
    Basic functions used on forms:
    override form_get and form_post
    """

    form_template = 'appbuilder/general/model/edit.html'
    """ Widgets """
    edit_widget = FormWidget

    form_title = 'Form Title'
    form_columns = []
    form = None
    redirect_url = '/'

    @expose("/form", methods=['GET'])
    @has_access
    def this_form_get(self):
        form = self.form.refresh()
        self.form_get(form)
        widgets = self._get_edit_widget(form = form)
        return render_template(
                                                self.form_template,
                                                title = self.form_title,
                                                widgets = widgets,
                                                baseapp = self.baseapp
                    )

    def form_get(self, form):
        pass

    @expose("/form", methods=['POST'])
    @has_access
    def this_form_post(self):
        form = self.form.refresh()
        if form.validate_on_submit():
            self.form_post(form)
            return redirect(self._get_redirect())
        else:
            widgets = self._get_edit_widget(form = form)
            return render_template(
                    self.form_template,
                    title = self.form_title,
                    widgets = widgets,
                    baseapp = self.baseapp
                    )

    def form_post(self, form):
        pass

    def _get_edit_widget(self, form = None, exclude_cols = [], widgets = {}):
        widgets['edit'] = self.edit_widget(route_base = self.route_base,
                                                form = form,
                                                exclude_cols = exclude_cols,
                                                )
        return widgets



class BaseCRUDView(BaseView):
    route_base = '/general'
    datamodel = None
    related_views = []
    redirect_url = route_base + '/list'

    """ Titles """
    list_title = ""
    show_title = ""
    add_title = ""
    edit_title = ""

    """ Include Columns """
    list_columns = []
    show_columns = []
    add_columns = []
    edit_columns = []
    order_columns = []
    search_columns = []

    label_columns = {}
    description_columns = {}

    """ fieldsets [(<'TITLE'|None>, {'fields':[<F1>,<F2>,...]}),....] """
    show_fieldsets = []
    add_fieldsets = []
    edit_fieldsets = []

    """ Forms """
    add_form = None
    edit_form = None
    search_form = None

    validators_columns = {}

    """ Template Layouts """
    list_template = 'appbuilder/general/model/list.html'
    edit_template = 'appbuilder/general/model/edit.html'
    add_template = 'appbuilder/general/model/add.html'
    show_template = 'appbuilder/general/model/show.html'

    """ Widgets """
    list_widget = ListWidget
    edit_widget = FormWidget
    add_widget = FormWidget
    show_widget = ShowWidget
    search_widget = SearchWidget

    
    """ Additional Widgets """
    show_additional_links = []

    
    def _init_forms(self):
        
        conv = GeneralModelConverter(self.datamodel)        
        if not self.add_form:
            self.add_form = conv.create_form(self.label_columns,
                    self.description_columns,
                    self.validators_columns,
                    self.add_columns)
        if not self.edit_form:
            self.edit_form = conv.create_form(self.label_columns,
                    self.description_columns,
                    self.validators_columns,
                    self.edit_columns)        
        if not self.search_form:
            self.search_form = conv.create_form(self.label_columns,
                    self.description_columns,
                    self.validators_columns,
                    self.search_columns)


    def _init_vars(self):
        if self.show_fieldsets:
            self.show_columns = []
            for fieldset_item in self.show_fieldsets:                
                self.show_columns = self.show_columns + list(fieldset_item[1].get('fields'))                
        if self.add_fieldsets:
            self.add_columns = []
            for fieldset_item in self.add_fieldsets:
                self.add_columns = self.add_columns + list(fieldset_item[1].get('fields'))
        else:
            if not self.add_columns:
                self.add_columns = self.datamodel.get_columns_list()
        if self.edit_fieldsets:
            self.edit_columns = []
            for fieldset_item in self.edit_fieldsets:
                self.edit_columns = self.edit_columns + list(fieldset_item[1].get('fields'))
        else:
            if not self.edit_columns:
                self.edit_columns = self.datamodel.get_columns_list()
                        
        
    def __init__(self, **kwargs):
        self._init_vars()
        self._init_forms()
        super(BaseCRUDView, self).__init__(**kwargs)
    

    """
    --------------------------
            WIDGET METHODS
    --------------------------
    """
    def _get_related_list_widget(self, item, related_view, 
                                filters={}, order_column='', order_direction=''):

        fk = related_view.datamodel.get_related_fk(self.datamodel.obj)
        filters[fk] = item
        return related_view._get_list_widget(filters = filters, 
                    order_column = order_column, order_direction = order_direction)
        
    def _get_related_list_widgets(self, item, filters = {}, order_column='', order_direction='', 
                                widgets = {}, **args):
        widgets['related_lists'] = []
        for view in self.related_views:
            widgets['related_lists'].append(self._get_related_list_widget(item, view, filters, order_column, order_direction).get('list'))
        return widgets
    
    def _get_list_widget(self, filters={}, order_column='', order_direction='', widgets = {}, **args):

        lst = self.datamodel.query(filters, order_column, order_direction)
        pks = self.datamodel.get_keys(lst)
        widgets['list'] = self.list_widget(route_base = self.route_base,
                                                label_columns = self.label_columns,
                                                include_columns = self.list_columns,
                                                value_columns = self.datamodel.get_values(lst, self.list_columns),
                                                order_columns = self.order_columns,
                                                pks = pks,
                                                filters = filters,
                                                generalview_name = self.__class__.__name__
                                                )
        return widgets

    def _get_search_widget(self, form = None, exclude_cols = [], widgets = {}):
        widgets['search'] = self.search_widget(route_base = self.route_base,
                                                form = form,
                                                exclude_cols = exclude_cols,
                                                )
        return widgets


    def _get_show_widget(self, id, widgets = {}):
        item = self.datamodel.get(id)
        widgets['show'] = self.show_widget(route_base = self.route_base,
                                                label_columns = self.label_columns,
                                                include_columns = self.show_columns,
                                                value_columns = self.datamodel.get_values_item(item, self.show_columns),
                                                additional_links = self.show_additional_links,
                                                fieldsets = self.show_fieldsets
                                                )
        return widgets


    def _get_add_widget(self, form = None, exclude_cols = [], widgets = {}):
        widgets['add'] = self.edit_widget(route_base = self.route_base,
                                                form = form,
                                                exclude_cols = exclude_cols,
                                                fieldsets = self.add_fieldsets
                                                )
        return widgets

    def _get_edit_widget(self, form = None, exclude_cols = [], widgets = {}):
        widgets['edit'] = self.edit_widget(route_base = self.route_base,
                                                form = form,
                                                exclude_cols = exclude_cols,
                                                fieldsets = self.edit_fieldsets
                                                )
        return widgets



    def debug(self):

        print self.__class__.__name__, "SHOW FS", self.show_fieldsets
        print self.__class__.__name__, "SHOW COL", self.show_columns
        print self.__class__.__name__, "ADD FS", self.add_fieldsets
        print self.__class__.__name__, "ADD COL", self.add_columns
        print self.__class__.__name__, "EDIT FS", self.edit_fieldsets
        print self.__class__.__name__, "EDIT COL", self.edit_columns
        print self.__class__.__name__, "LIST COL", self.list_columns

    """
    -----------------------------------
     Methods to override
    -----------------------------------
    """
    @classmethod
    def pre_update(self, item):
        pass

    @classmethod
    def post_update(self, item):
        pass

    @classmethod
    def pre_add(self, item):
        pass

    @classmethod
    def post_add(self, item):
        pass

    @classmethod
    def pre_delete(self, item):
        pass

    @classmethod
    def post_delete(self, item):
        pass



class GeneralView(BaseCRUDView):
    """
    General View Class
    Responsible to handle all CRUD events
    List, Edit, Add, Show based on models
    """

    def __init__(self, **kwargs):
        super(GeneralView, self).__init__(**kwargs)

    """
    --------------------------------
            LIST
    --------------------------------
    """
    
    @expose('/list/', methods=['GET', 'POST'])
    @has_access
    def list(self):

        form = self.search_form.refresh()
        search_form = self.search_form(request.form)

        order_column, order_direction = self._get_order_args()

        filters = {}
        filters = self._get_filter_args(filters)
        filters = self._get_dict_from_form(search_form, filters)
        if (filters != {}):
            item = self.datamodel.obj()
            for filter_key in filters:
                try:
                    rel_obj = self.datamodel.get_related_obj(filter_key, filters.get(filter_key))
                    setattr(item, filter_key, rel_obj)
                    search_form = self.search_form(obj = item)
                    filters[filter_key] = rel_obj
                except:
                    setattr(item, filter_key, filters.get(filter_key))
                    search_form = self.search_form(obj = item)

        widgets = self._get_list_widget(filters, order_column, order_direction)
        widgets = self._get_search_widget(form = search_form, widgets = widgets)

        return render_template(self.list_template,
                                        title = self.list_title,
                                        widgets = widgets,
                                        baseapp = self.baseapp)



    """
    --------------------------------
            SHOW
    --------------------------------
    """
    @expose('/show/<int:id>', methods=['GET'])
    @has_access
    def show(self, id):

        widgets = self._get_show_widget(id)
        item = self.datamodel.get(id)
        widgets = self._get_related_list_widgets(item, filters = {}, order_column='', order_direction='', widgets = widgets)
        return render_template(self.show_template,
                           title = self.show_title,
                           widgets = widgets,
                           baseapp = self.baseapp,
                           related_views = self.related_views)



    """
    ---------------------------
            ADD
    ---------------------------
    """
    @expose('/add', methods=['GET', 'POST'])
    @has_access
    def add(self):

        filters = self._get_filter_args(filters={})

        form = self.add_form.refresh()
        exclude_cols = self.datamodel.get_relation_filters(filters)

        if form.validate_on_submit():
            item = self.datamodel.obj()
            form.populate_obj(item)
            for filter_key in exclude_cols:
                rel_obj = self.datamodel.get_related_obj(filter_key, filters.get(filter_key))
                setattr(item, filter_key, rel_obj)
                        
            self.pre_add(item)
            self.datamodel.add(item)
            self.post_add(item)
            return redirect(self._get_redirect())
        else:
            widgets = self._get_add_widget(form = form, exclude_cols = exclude_cols)
            return render_template(
                                                    self.add_template,
                                                    title = self.add_title,
                                                    widgets = widgets,
                                                    baseapp = self.baseapp
                                                    )    

    """
    ---------------------------
            EDIT
    ---------------------------
    """
    @expose('/edit/<int:pk>', methods=['GET', 'POST'])
    @has_access
    def edit(self, pk = 0):

        item = self.datamodel.get(pk)
        filters = self._get_filter_args(filters={})
        exclude_cols = self.datamodel.get_relation_filters(filters)

        if request.method == 'POST':
            form = self.edit_form(request.form)
            form = form.refresh(obj=item)
            if form.validate():
                form.populate_obj(item)

                for filter_key in exclude_cols:
                    rel_obj = self.datamodel.get_related_obj(filter_key, filters.get(filter_key))
                    setattr(item, filter_key, rel_obj)
                
                self.pre_update(item)
                self.datamodel.edit(item)
                self.post_update(item)
                return redirect(self._get_redirect())
            else:
                widgets = self._get_edit_widget(form = form, exclude_cols = exclude_cols)
                widgets = self._get_related_list_widgets(item, filters = {}, order_column='', order_direction='', widgets = widgets)
                return render_template(self.edit_template,
                        title = self.edit_title,
                        widgets = widgets,
                        baseapp = self.baseapp,
                        related_views = self.related_views)
        else:
            form = self.edit_form(obj=item)
            form = form.refresh(obj=item)
            widgets = self._get_edit_widget(form = form, exclude_cols = exclude_cols)
            widgets = self._get_related_list_widgets(item, filters = {}, order_column='', order_direction='', widgets = widgets)
            return render_template(self.edit_template,
                            title = self.edit_title,
                            widgets = widgets,
                            baseapp = self.baseapp,
                            related_views = self.related_views)


    """
    ---------------------------
            DELETE
    ---------------------------
    """
    @expose('/delete/<int:id>')
    @has_access
    def delete(self, id):
        item = self.datamodel.get(id)
        
        self.pre_delete(item)
        self.datamodel.delete(item)
        self.post_delete(item)        
        return redirect(self._get_redirect())


    @expose('/download/<string:filename>')
    @has_access
    def download(self, filename):
        return send_file(self.baseapp.app.config['UPLOAD_FOLDER'] + filename, 
                    attachment_filename = uuid_originalname(filename), 
                    as_attachment = True)
        

class FileCRUDView(BaseCRUDView):
    pass

    


class AdditionalLinkItem():
    label = ""
    href = ""
    icon = ""

    def __init__(self, label="", href="", icon=""):
        self.label = label
        self.href = href
        self.icon = icon



from flask import render_template
from sqlalchemy.ext.serializer import loads, dumps

from sqlalchemy.ext.serializer import loads, dumps

from widgets import ChartWidget
from ..security.decorators import has_access
from ..views import BaseView, expose


class ChartView(BaseView):
    
    chart_template = 'appbuilder/general/charts/chart.html'
    chart_widget = ChartWidget
    chart_title = 'Chart'
    chart_type = 'PieChart'
    chart_3d = 'true'
    width = 400
    height = '400px'
    group_by = ''
    label_columns = []
    group_by_columns = []
    datamodel = None
    
    def _get_chart_widget(self, value_columns = [], widgets = {}):        
        widgets['chart'] = self.chart_widget(route_base = self.route_base,
                                             chart_title = self.chart_title, 
                                             chart_type = self.chart_type,
                                             chart_3d = self.chart_3d, 
                                                value_columns = value_columns)
        return widgets
    
    @expose('/chart/')
    @has_access
    def show(self):
        group_by = self._get_group_by_args()
        if group_by == '':
            group_by = self.group_by_columns[0]
        value_columns = self.datamodel.query_simple_group(group_by)
        widgets = self._get_chart_widget(value_columns = value_columns)
        return render_template(self.chart_template, route_base = self.route_base, 
                                                title = self.chart_title,
                                                label_columns = self.label_columns, 
                                                group_by_columns = self.group_by_columns,
                                                height = self.height,
                                                widgets = widgets, 
                                                baseapp = self.baseapp)
    
    



from flask import render_template, flash, redirect, session, url_for, request, g
from flask.ext.login import login_user, logout_user, current_user, login_required
from flask.ext.wtf import Required, Length, validators, EqualTo, PasswordField
from flask.ext.babel import gettext, lazy_gettext

from flask.ext.appbuilder.views import *
from models import *
from ..forms import BS3PasswordFieldWidget
from forms import *


from app import app, db, lm, oid
from flask.ext.appbuilder.models.datamodel import SQLAModel
from config import AUTH_TYPE, APP_NAME, APP_THEME


AUTH_OID = 0
AUTH_DB = 1
AUTH_LDAP = 2



#----------------------------- DEFS --------------------------------------
#-------------------------------------------------------------------------

class PermissionGeneralView(GeneralView):
    route_base = '/permissions'
    
    datamodel = SQLAModel(Permission, db.session)

    list_title = lazy_gettext('List Base Permissions')
    show_title = lazy_gettext('Show Base Permission')
    add_title = lazy_gettext('Add Base Permission')
    edit_title = lazy_gettext('Edit Base Permission')

    label_columns = {'name':lazy_gettext('Name')}
    list_columns = ['name']
    show_columns = ['name']
    order_columns = ['name']
    search_columns = ['name']

class ViewMenuGeneralView(GeneralView):
    route_base = '/viewmenus'
    
    datamodel = SQLAModel(ViewMenu, db.session)

    list_title = lazy_gettext('List View Menus')
    show_title = lazy_gettext('Show View Menu')
    add_title = lazy_gettext('Add View Menu')
    edit_title = lazy_gettext('Edit View Menu')

    label_columns = {'name':lazy_gettext('Name')}
    list_columns = ['name']
    show_columns = ['name']
    order_columns = ['name']
    search_columns = ['name']

class PermissionViewGeneralView(GeneralView):
    route_base = '/permissionviews'
    
    datamodel = SQLAModel(PermissionView , db.session)

    list_title = lazy_gettext('List Permissions on Views/Menus')
    show_title = lazy_gettext('Show Permission on Views/Menus')
    add_title = lazy_gettext('Add Permission on Views/Menus')
    edit_title = lazy_gettext('Edit Permission on Views/Menus')

    label_columns = {'permission':lazy_gettext('Permission'), 'view_menu': lazy_gettext('View/Menu')}
    list_columns = ['permission', 'view_menu']
    show_columns = ['permission', 'view_menu']
    order_columns = ['permission', 'view_menu']
    search_columns = ['permission', 'view_menu']


class ResetPasswordView(SimpleFormView):

    route_base = '/resetpassword'
    

    form = ResetPasswordForm
    form_title = lazy_gettext('Reset Password Form')
    form_columns = ['password','conf_password']
    redirect_url = '/'

    message = lazy_gettext('Password Changed')

    def form_post(self, form):
        user = db.session.query(User).get(g.user.id)
        user.password = form.password.data
        db.session.commit()
        flash(unicode(self.message),'info')


class UserGeneralView(GeneralView):
    route_base = '/users'
    datamodel = SQLAModel(User, db.session)
    

    list_title = lazy_gettext('List Users')
    show_title = lazy_gettext('Show User')
    add_title = lazy_gettext('Add User')
    edit_title = lazy_gettext('Edit User')

    label_columns = {'get_full_name':lazy_gettext('Full Name'),
                    'first_name':lazy_gettext('First Name'),
                    'last_name':lazy_gettext('Last Name'),
                    'username':lazy_gettext('User Name'),
                    'password':lazy_gettext('Password'),
                    'active':lazy_gettext('Is Active?'),
                    'email':lazy_gettext('EMail'),
                    'role':lazy_gettext('Role')}
    description_columns = {'first_name':lazy_gettext('Write the user first name or names'),
                    'last_name':lazy_gettext('Write the user last name'),
        'username':lazy_gettext('Username valid for authentication on DB or LDAP, unused for OID auth'),
                    'password':lazy_gettext('Please use a good password policy, this application does not check this for you'),
                    'active':lazy_gettext('Its not a good policy to remove a user, just make it inactive'),
                    'email':lazy_gettext('The users email, this will also be used for OID auth'),
                    'role':lazy_gettext('The user role on the application, this will associate with a list of permissions'),
                    'conf_password':lazy_gettext('Please rewrite the users password to confirm')}
    list_columns = ['get_full_name', 'username', 'email','active', 'role']
    show_columns = ['first_name','last_name','username', 'active', 'email','role']
    order_columns = ['username', 'email']
    search_columns = ['first_name','last_name', 'username', 'email']

    add_columns = ['first_name','last_name','username', 'active', 'email','role']
    edit_columns = ['first_name','last_name','username', 'active', 'email','role']

    user_info_title = lazy_gettext("Your user information")
    lnk_reset_password = lazy_gettext("Reset Password")

    show_additional_links = []

    @expose('/userinfo/')
    def userinfo(self):
        item = g.user

        additional_links = None


        if AUTH_TYPE == AUTH_DB:
            if not self.show_additional_links:
                self.show_additional_links.append(AdditionalLinkItem(self.lnk_reset_password,"/resetpassword/form","lock"))

        widgets = self._get_show_widget(item.id)

        return render_template(self.show_template,
                           title = self.user_info_title,
                           widgets = widgets,
                           baseapp = self.baseapp,
                           )



    def _init_forms(self):
        super(UserGeneralView, self)._init_forms()
        if AUTH_TYPE == AUTH_DB:
            self.add_form.password = PasswordField('Password', 
                                                   description=self.description_columns['password'],
                                                   widget=BS3PasswordFieldWidget())
            self.add_form.conf_password = PasswordField('Confirm Password',
                                            default=self.add_form.password,
                                            description=self.description_columns['conf_password'],
                                            validators=[EqualTo('password',message=u'Passwords must match')],
                                            widget=BS3PasswordFieldWidget())
            if 'password' not in self.add_columns:
                self.add_columns = self.add_columns + ['password', 'conf_password']
        else:
            self.add_form.password = None



class RoleGeneralView(GeneralView):
    route_base = '/roles'
    
    datamodel = SQLAModel(Role, db.session)

    related_views = [PermissionViewGeneralView(), UserGeneralView()]

    list_title = lazy_gettext('List Roles')
    show_title = lazy_gettext('Show Role')
    add_title = lazy_gettext('Add Role')
    edit_title = lazy_gettext('Edit Role')

    label_columns = { 'name':lazy_gettext('Name'),'permissions':lazy_gettext('Permissions') }
    list_columns = ['name','permissions']
    show_columns = ['name','permissions']
    order_columns = ['name']
    search_columns = ['name']



class AuthView(BaseView):

    route_base = ''
    login_db_template = 'appbuilder/general/security/login_db.html'
    login_oid_template = 'appbuilder/general/security/login_oid.html'
    
    invalid_login_message = lazy_gettext('Invalid login. Please try again.')
    
    title = lazy_gettext('Sign In')

    @expose('/login/', methods = ['GET', 'POST'])
    @oid.loginhandler
    def login(self):
        if AUTH_TYPE == AUTH_OID: return self._login_oid()
        if AUTH_TYPE == AUTH_DB: return self._login_db()

    @expose('/logout/')
    def logout(self):
        logout_user()
        return redirect('/')


    def _login_db(self):
        if g.user is not None and g.user.is_authenticated():
            return redirect('/')
        form = LoginForm_db()
        if form.validate_on_submit():
            if form.username.data is None or form.username.data == "":
                flash(unicode(self.invalid_login_message),'warning')
                return redirect('/login')
            user = User.query.filter_by(username = form.username.data, password = form.password.data).first()
            if user is None or (not user.is_active()):
                flash(unicode(self.invalid_login_message),'warning')
                return redirect('/login')
            login_user(user, remember = False)
            return redirect('/')
        return render_template(self.login_db_template,
                                                title = self.title,
                                                form = form,
                                                baseapp = self.baseapp
                                                )

    def _login_oid(self):
        if g.user is not None and g.user.is_authenticated():
            return redirect('/')
        form = LoginForm_oid()
        if form.validate_on_submit():
            session['remember_me'] = form.remember_me.data
            return oid.try_login(form.openid.data, ask_for = ['email'])
        return render_template(self.login_oid_template,
                title = self.title,
                form = form,
                providers = app.config['OPENID_PROVIDERS'],
                baseapp = self.baseapp
                )


def _after_login_oid(resp):
    if resp.email is None or resp.email == "":
        flash(gettext('Invalid login. Please try again.'),'warning')
        return redirect('appbuilder/general/security/login_oid.html')
    user = User.query.filter_by(email = resp.email).first()
    if user is None:
        flash(gettext('Invalid login. Please try again.'),'warning')
        return redirect('appbuilder/general/security/login_oid.html')
    remember_me = False
    if 'remember_me' in session:
        remember_me = session['remember_me']
        session.pop('remember_me', None)

    login_user(user, remember = remember_me)
    return redirect('/')


@oid.after_login
def _after_login(resp):
    if AUTH_TYPE == AUTH_OID: return _after_login_oid(resp)


@lm.user_loader
def load_user(id):
    return User.query.get(int(id))


@app.before_request
def before_request():
    g.user = current_user



from flask import redirect, request, session
from flask.ext.babel import refresh
from app import babel
from config import BABEL_DEFAULT_LOCALE
from ..views import BaseView, expose


class LocaleView(BaseView):
    route_base = '/lang'

    @expose('/<string:locale>')
    def index(self, locale):
        session['locale'] = locale
        refresh()
        return redirect(self._get_redirect())


@babel.localeselector
def get_locale():
    locale = session.get('locale')
    if locale:
        return locale
    session['locale'] = BABEL_DEFAULT_LOCALE
    return session['locale']



import os
import os.path as op

from werkzeug import secure_filename
from werkzeug.datastructures import FileStorage
from config import UPLOAD_FOLDER, IMG_UPLOAD_FOLDER, IMG_UPLOAD_URL
import uuid

try:
    from PIL import Image, ImageOps
except ImportError:
    Image = None
    ImageOps = None
    

class FileManager(object):
    
    def __init__(self, base_path=UPLOAD_FOLDER,
                    relative_path="",
                    namegen = None,
                    allowed_extensions=None,
                    permission=0o666, **kwargs):
        
        self.base_path = base_path
        self.relative_path = relative_path
        self.namegen = namegen or uuid_namegen
        self.allowed_extensions = allowed_extensions
        self.permission = permission
        self._should_delete = False
        
    
    def is_file_allowed(self, filename):
        if not self.allowed_extensions:
            return True
        return ('.' in filename and
                filename.rsplit('.', 1)[1].lower() in self.allowed_extensions)
    
    def generate_name(self, obj, file_data):
        return self.namegen(obj, file_data)

    def get_path(self, filename):
        if not self.base_path:
            raise ValueError('FileUploadField field requires base_path to be set.')
        return op.join(self.base_path, filename)

    def delete_file(self, filename):
        path = self.get_path(filename)
        if op.exists(path):
            os.remove(path)

    def save_file(self, data, filename):
        path = self.get_path(filename)
        if not op.exists(op.dirname(path)):
            os.makedirs(os.path.dirname(path), self.permission)
        data.save(path)
        return filename

    
    
class ImageManager(FileManager):
    
    keep_image_formats = ('PNG',)
    
    def __init__(self,base_path=IMG_UPLOAD_FOLDER,
                    max_size=(300,200,True),
                    namegen = None,
                    relative_path=IMG_UPLOAD_URL,
                    allowed_extensions=None,
                    thumbgen=None, thumbnail_size=None,
                    permission=0o666,
                    **kwargs):

        # Check if PIL is installed
        if Image is None:
            raise Exception('PIL library was not found')

        self.max_size = max_size
        self.thumbnail_fn = thumbgen or thumbgen_filename
        self.thumbnail_size = thumbnail_size
        self.image = None
        
        if not allowed_extensions:
            allowed_extensions = ('gif', 'jpg', 'jpeg', 'png', 'tiff')

        super(ImageManager, self).__init__(base_path=base_path,
                                        relative_path=relative_path,
                                        namegen=namegen,
                                        allowed_extensions=allowed_extensions,
                                        permission=permission,
                                        **kwargs)

    def get_url(self, filename):
        return self.relative_path + filename
    
    # Deletion
    def delete_file(self, filename):
        super(ImageManager, self).delete_file(filename)

        self.delete_thumbnail(filename)

    def delete_thumbnail(self, filename):
        path = self.get_path(self.thumbnail_fn(filename))

        if op.exists(path):
            os.remove(path)

    # Saving
    def save_file(self, data, filename):
        if data and isinstance(data, FileStorage):
            try:
                self.image = Image.open(data)
            except Exception as e:
                raise ValidationError('Invalid image: %s' % e)
        
        path = self.get_path(filename)

        if not op.exists(op.dirname(path)):
            os.makedirs(os.path.dirname(path), self.permission)

        # Figure out format
        filename, format = self.get_save_format(filename, self.image)
        print "IMG SAVE: ", filename,":", format, ":", self.max_size
        if self.image and (self.image.format != format or self.max_size):
            if self.max_size:
                image = self.resize(self.image, self.max_size)
            else:
                image = self.image
            self.save_image(image, self.get_path(filename), format)
        else:
            data.seek(0)
            data.save(path)
        self.save_thumbnail(data, filename, format)

        return filename

    def save_thumbnail(self, data, filename, format):
        if self.image and self.thumbnail_size:
            path = self.get_path(self.thumbnail_fn(filename))

            self.save_image(self.resize(self.image, self.thumbnail_size),
                             path,
                             format)

    def resize(self, image, size):
        print "RESIZE"
        (width, height, force) = size

        if image.size[0] > width or image.size[1] > height:
            if force:
                print "FORCE"
                return ImageOps.fit(self.image, (width, height), Image.ANTIALIAS)
            else:
                thumb = self.image.copy()
                thumb.thumbnail((width, height), Image.ANTIALIAS)
                return thumb

        return image

    def save_image(self, image, path, format='JPEG'):
        if image.mode not in ('RGB', 'RGBA'):
            image = image.convert('RGBA')

        with open(path, 'wb') as fp:
            image.save(fp, format)

    def get_save_format(self, filename, image):
        if image.format not in self.keep_image_formats:
            name, ext = op.splitext(filename)
            filename = '%s.jpg' % name
            return filename, 'JPEG'
        return filename, image.format


def uuid_namegen(obj, file_data):
    return str(uuid.uuid1()) + '_sep_' + file_data.filename
    

def uuid_originalname(uuid_filename):
    return uuid_filename.split('_sep_')[1]
    
def thumbgen_filename(filename):
    name, ext = op.splitext(filename)
    return '%s_thumb%s' % (name, ext)
    



from flask.ext.appbuilder.widgets import RenderTemplateWidget

class ChartWidget(RenderTemplateWidget):

    template = 'appbuilder/general/widgets/chart.html'

    route_base = ''
    chart_title = ''
    value_columns = []
    chart_type = ''
    chart_3d = 'false'
    
    
    def __init__(self, route_base = '',
                chart_title = '',
                chart_type = '',
                chart_3d ='false',
                value_columns = []):
                
        self.route_base = route_base
        self.chart_title = chart_title
        self.chart_type = chart_type
        self.chart_3d = chart_3d
        self.value_columns = value_columns
        
    def __call__(self, **kwargs):
        kwargs['route_base'] = self.route_base
        kwargs['chart_title'] = self.chart_title
        kwargs['chart_type'] = self.chart_type
        kwargs['chart_3d'] = self.chart_3d 
        kwargs['value_columns'] = self.value_columns
        
        return super(ChartWidget, self).__call__(**kwargs)




from .security.models import PermissionView

class MenuItem(object):

    name = ""
    href = "#"
    icon = ""
    childs = []

    def __init__(self, name, href="#", icon="", childs=[]):
        self.name = name
        self.href = href
        self.icon = icon
        if (self.childs):
            self.childs = childs
        else: self.childs = []

    def __repr__(self):
        return self.name + ' href=' + self.href

class Menu(object):

    menu = []

    def get_list(self):
        return self.menu

    # Mal programado para rever
    def find_category(self, category):
        for i in self.menu:
            if i.name == category:
                return i
            else:
                for j in i.childs:
                    if j.name == category:
                        return j


    def add_category(self, category, icon="", parent_category=""):
        if parent_category == "":
            self.menu.append(MenuItem(name=category, icon = icon))
        else:
            self.find_category(parent_category).childs.append(MenuItem(name=category, icon = icon))
        pvm = PermissionView()
        try:
            pvm = pvm.add_menu_permissions(category)
        except:
            print "Menu add_category Erro: Maybe db not created"


    def add_link(self, name, href="#", icon="", parent_category=""):
        menu_item = self.find_category(parent_category)
        if menu_item:
            menu_item.childs.append(MenuItem(name=name, href=href, icon = icon))
        else:
            self.add_category(category=parent_category)
            self.find_category(parent_category).childs.append(MenuItem(name=name, href=href, icon = icon))
        pvm = PermissionView()
        try:
            pvm = pvm.add_menu_permissions(name)
        except:
            print "Menu add_category Erro: Maybe db not created"


    def add_separator(self, parent_category=""):
        self.find_category(parent_category).childs.append(MenuItem("-"))



from flask import Markup
from hashlib import md5
from app import db
from config import AUTH_ROLE_ADMIN, AUTH_ROLE_PUBLIC
from werkzeug.security import generate_password_hash, check_password_hash
from ..models.mixins import BaseMixin

def is_menu_public(item):
    role = Role.query.filter_by(name = AUTH_ROLE_PUBLIC).first()
    lst = role.permissions
    if lst:
        for i in lst:
            if item == i.view_menu.name:
                return  True
        return False
    else: return False

def is_item_public(permission_name, generalview_name):
    role = Role.query.filter_by(name = AUTH_ROLE_PUBLIC).first()
    lst = role.permissions
    if lst:
        for i in lst:
            if (generalview_name == i.view_menu.name) and (permission_name == i.permission.name):
                return True
        return False
    else: return False



class Permission(BaseMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), unique = True, nullable=False)

    def add_unique(self, name):
        perm = Permission.query.filter_by(name = name).first()
        if perm == None:
            perm = Permission()
            perm.name = name
            db.session.add(perm)
            db.session.commit()
            return perm
        return perm

    def __repr__(self):
        return self.name

class ViewMenu(BaseMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), unique = True, nullable=False)


    def add_unique(self, name):
        view_menu = ViewMenu.query.filter_by(name = name).first()
        if view_menu == None:
            view_menu = ViewMenu()
            view_menu.name = name
            db.session.add(view_menu)
            db.session.commit()
            return view_menu
        return view_menu

    def __eq__(self, other):
        return (isinstance(other, self.__class__)) and (self.name == other.name)

    def __neq__(self, other):
        return self.name != other.name

    def __repr__(self):
        return self.name



class PermissionView(BaseMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    permission_id = db.Column(db.Integer, db.ForeignKey('permission.id'))
    permission = db.relationship("Permission")
    view_menu_id = db.Column(db.Integer, db.ForeignKey('view_menu.id'))
    view_menu = db.relationship("ViewMenu")

    def add_unique(self, permission_name, view_menu_name):
        vm = ViewMenu()
        vm = vm.add_unique(view_menu_name)
        perm = Permission()
        perm = perm.add_unique(permission_name)
        pv = PermissionView()
        pv.view_menu_id, pv.permission_id = vm.id, perm.id
        db.session.add(pv)
        db.session.commit()
        print "Added Permission View " , str(pv)
        return pv


    def _find_permission(self, lst, item):
        for i in lst:
            if i.permission.name == item:
                return True
        return False

    def add_view_permissions(self, base_permissions, view_menu):
        view_menu_db = ViewMenu.query.filter_by(name = view_menu).first()
        if view_menu_db == None:
            view_menu_db = ViewMenu()
            view_menu_db = view_menu_db.add_unique(view_menu)
        lst = PermissionView.query.filter_by(view_menu_id = view_menu_db.id).all()
        # No permissions for this view
        if lst == []:
            for perm_str in base_permissions:
                pv = self.add_unique(perm_str, view_menu)
                role_admin = Role.query.filter_by(name = AUTH_ROLE_ADMIN).first()
                role_admin.add_unique_permission(pv)
        else:
            for item in base_permissions:
                if not self._find_permission(lst, item):
                    pv = self.add_unique(item, view_menu)
                    role_admin = Role.query.filter_by(name = AUTH_ROLE_ADMIN).first()
                    role_admin.add_unique_permission(pv)
            for item in lst:
                if item.permission.name not in base_permissions:
                    # perm to delete
                    pass

    def add_menu_permissions(self, view_menu):
        view_menu_db = ViewMenu.query.filter_by(name = view_menu).first()
        if view_menu_db == None:
            view_menu_db = ViewMenu()
            view_menu_db = view_menu_db.add_unique(view_menu)
        lst = PermissionView.query.filter_by(view_menu_id = view_menu_db.id).all()
        if lst == []:
            pv = self.add_unique('menu_access', view_menu)
            role_admin = Role.query.filter_by(name = AUTH_ROLE_ADMIN).first()
            role_admin.add_unique_permission(pv)

    def __repr__(self):
        return str(self.permission).replace('_',' ') + ' on ' + str(self.view_menu)

assoc_permissionview_role = db.Table('permission_view_role', db.Model.metadata,
        db.Column('permission_view_id', db.Integer, db.ForeignKey('permission_view.id')),
        db.Column('role_id', db.Integer, db.ForeignKey('role.id'))
        )


class Role(BaseMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(64), unique = True, nullable=False)
    permissions = db.relationship('PermissionView', secondary = assoc_permissionview_role, backref='role')


    def add_unique_permission(self, perm_view):
        if perm_view not in self.permissions:
            self.permissions.append(perm_view)
            db.session.merge(self)
            db.session.commit()
            print "Added Role " , str(perm_view) , " to Role " , self.name

    def __repr__(self):
        return self.name

class User(BaseMixin, db.Model):
    id = db.Column(db.Integer, primary_key=True)
    first_name = db.Column(db.String(64), nullable = False)
    last_name = db.Column(db.String(64), nullable = False)
    username = db.Column(db.String(32), unique=True, nullable = False)
    password = db.Column(db.String(32))
    active = db.Column(db.Boolean)
    email = db.Column(db.String(64))
    role_id = db.Column(db.Integer, db.ForeignKey('role.id'))
    role = db.relationship("Role")


    def has_menu_access(self, menu_name):
        lst = self.role.permissions
        if lst:
            for i in lst:
                if menu_name == i.view_menu.name:
                    return  True
            return False
        else: return False

    def has_permission_on_view(self, permission_name, generalview_name):
        lst = self.role.permissions
        if lst:
            for i in lst:
                if (generalview_name == i.view_menu.name) and (permission_name == i.permission.name):
                    return True
            return False
        else: return False

    @staticmethod
    def make_unique_nickname(nickname):
        if User.query.filter_by(nickname = nickname).first() == None:
            return nickname
        version = 2
        while True:
            new_nickname = nickname + str(version)
            if User.query.filter_by(nickname = new_nickname).first() == None:
                break
            version += 1
        return new_nickname

    def check_password(self, password):
        return check_password_hash(self.password, password)

    def is_authenticated(self):
        return True

    def is_active(self):
        return self.active

    def is_anonymous(self):
        return False

    def get_id(self):
        return unicode(self.id)

    def get_full_name(self):
        return self.first_name + " " + self.last_name

    def __repr__(self):
        return (self.get_full_name())



from flask import Blueprint
from flask.ext.babel import lazy_gettext
from flask.ext.babel import gettext as _gettext
from .security.views import AuthView, ResetPasswordView, UserGeneralView, RoleGeneralView, PermissionViewGeneralView, ViewMenuGeneralView, PermissionGeneralView, IndexView, PermissionView
from .babel.views import LocaleView
from menu import Menu

class BaseApp():

    lst_baseview = []
    app = None
    menu = None
    app_name = ""
    indexview = None
    
    static_folder = None
    static_url_path = None
    
    languages = None
    admin = None
    _gettext = _gettext

    """
    ------------------------------------
                 INIT
     Add menu with categories inserted
    #-----------------------------------
    """
    def __init__(self, app, menu = None, indexview = None, static_folder='static/appbuilder', static_url_path='/appbuilder'):
        self.menu = menu or Menu()
        self.app = app
        self.app_name = app.config['APP_NAME']
        self.app_theme = app.config['APP_THEME']
        self.languages = app.config['LANGUAGES']
        self.indexview = indexview or IndexView
        self.static_folder = static_folder
        self.static_url_path = static_url_path
        self._add_admin_views()
        

    def _add_admin_views(self, ):
        self.add_view_no_menu(self.indexview)
        self.add_view_no_menu(LocaleView)
        self.add_view_no_menu(AuthView)
        self.add_view_no_menu(ResetPasswordView)

        self.add_view(UserGeneralView, "List Users"
                                        ,"/users/list","user",
                                        "Security")
        self.add_view(RoleGeneralView, "List Roles","/roles/list","tags","Security")
        self.menu.add_separator("Security")
        self.add_view(PermissionViewGeneralView, "Base Permissions","/permissions/list","lock","Security")
        self.add_view(ViewMenuGeneralView, "Views/Menus","/viewmenus/list","list-alt","Security")
        self.add_view(PermissionGeneralView, "Permission on Views/Menus","/permissionviews/list","lock","Security")

        bp = Blueprint('baseapp', __name__, url_prefix='/static',
                template_folder='templates', static_folder = self.static_folder, static_url_path = self.static_url_path)
        self.app.register_blueprint(bp)

    def add_view(self, baseview, name, href, icon, category):
        print "Registering:", category,".", name, "at", href
        self.menu.add_link(name, href, icon, category)
        if baseview not in self.lst_baseview:
            baseview.baseapp = self
            self.lst_baseview.append(baseview)
            self.register_blueprint(baseview)
            self._add_permission(baseview)

    def add_view_no_menu(self, baseview, endpoint = None, static_folder = None):
        if baseview not in self.lst_baseview:
            baseview.baseapp = self
            self.lst_baseview.append(baseview)
            self.register_blueprint(baseview, endpoint = endpoint, static_folder = static_folder)
            self._add_permission(baseview)

    def _add_permission(self, baseview):
        pvm = PermissionView()
        bv = baseview()
        try:
            pvm.add_view_permissions(bv.base_permissions, bv.__class__.__name__)
        except:
            print "General _add_permission Error: DB not created?"
        bv = None
        pvm = None

    def register_blueprint(self, baseview, endpoint = None, static_folder = None):
        self.app.register_blueprint(baseview().create_blueprint(self,  endpoint = endpoint, static_folder = static_folder))



from app import app, db
from flask.ext.appbuilder.security.models import is_menu_public, is_item_public
from flask.ext.appbuilder.models.datamodel import SQLAModel
from flask import g, request
from flask.ext.login import current_user

@app.template_filter('link_order')
def link_order_filter(s):

    if (request.args.get('order_column') == s):
        lststr = request.path.split('?')
        if (request.args.get('order_direction') == 'asc'):
            return lststr[0] + '?order_column=' + s + '&order_direction=desc'
        else:
            return  request.path + '?order_column=' + s + '&order_direction=asc'
    else:
        return  request.path + '?order_column=' + s + '&order_direction=asc'


@app.template_filter('get_link_next')
def get_link_next_filter(s):
    print "NEXT = ", request.args.get('next')
    return request.args.get('next')


@app.template_filter('set_link_filters')
def add_link_filters_filter(path, filters):
    lnkstr = path
    for _filter in filters:
        try:
            datamodel = SQLAModel(filters.get(_filter))
            pk_name = datamodel.get_pk_name()
            pk_value = getattr(filters.get(_filter), pk_name)
            lnkstr = lnkstr + '&_flt_' + _filter + '=' + str(pk_value)
        except:
            pass
    return lnkstr


@app.template_filter('get_link_order')
def get_link_order_filter(s):
    if request.args.get('order_column') == s:
        order_direction = request.args.get('order_direction')
        if (order_direction):
            if (order_direction == 'asc'):
                return 2
            else:
                return 1
    else:
        return 0

@app.template_filter('get_attr')
def get_attr_filter(obj, item):
    return getattr(obj, item)


@app.template_filter('is_menu_visible')
def is_menu_visible(item):
    if current_user.is_authenticated():
        if is_menu_public(item) or g.user.has_menu_access(item.name):
            return True
        else:
            return False
    else:
        if is_menu_public(item.name):
            return True
        else:
            return False

@app.template_filter('is_item_visible')
def is_item_visible(permission, item):
    if current_user.is_authenticated():
        if g.user.has_permission_on_view(permission, item):
            return True
        else:
            return False
    else:
        if is_item_public(permission, item):
            return True
        else:
            return False



from sqlalchemy.ext.declarative import declared_attr
from sqlalchemy import Table, Column, Integer, String, MetaData, ForeignKey, DateTime
import sqlalchemy.types as types
from sqlalchemy.types import String
# from app import db
from flask import g
import uuid
import datetime

class FileColumn(types.TypeDecorator):
    """
    Extends SQLA to support a File Column
    usefull to auto form generation.
    
    """
    impl = types.Text
    

class ImageColumn(types.TypeDecorator):
    impl = types.Text

        

class BaseMixin(object):
    __table_args__ = {'extend_existing': True}


class AuditMixin(BaseMixin):
    """
    AuditMixin
    Mixin for models, add 4 cols to stamp, time and user on
    creation and modification
    """
    created_on = Column(DateTime, default=datetime.datetime.now,nullable=False)
    changed_on = Column(DateTime, default=datetime.datetime.now,
                            onupdate=datetime.datetime.now,nullable=False)

    @declared_attr
    def created_by_fk(cls):
        return Column(Integer, ForeignKey('user.id'),
                default=cls.get_user_id,nullable=False)

    @declared_attr
    def created_by(cls):
        return relationship("User", primaryjoin='%s.created_by_fk == User.id'%cls.__name__)

    @declared_attr
    def changed_by_fk(cls):
        return Column(Integer, ForeignKey('user.id'),
                default=cls.get_user_id,onupdate=cls.get_user_id,nullable=False)

    @declared_attr
    def changed_by(cls):
        return relationship("User", primaryjoin='%s.changed_by_fk == User.id'%cls.__name__)


    @classmethod
    def get_user_id(cls):
        return g.user.id



from flask.ext.babel import lazy_gettext as _

"""
This Module is not used.
Just use it to automate Babel extraction
"""

auto_translations_import = [
_("Security"),
_("List Users"),
_("Base Permissions"),
_("Views/Menus"),
_("Permission on Views/Menus"),
_("Search"),
_("Back"),
_("Save"),
_("This field is required."),
_("Not a valid date value"),
_("No records found")
]



import os
import os.path as op

from werkzeug import secure_filename
from werkzeug.datastructures import FileStorage

from wtforms import ValidationError, fields
from wtforms.widgets import HTMLString, html_params
from wtforms.fields.core import _unset_value
from flask.ext.babel import gettext
from .filemanager import ImageManager, FileManager



"""
    Based and thanks to https://github.com/mrjoes/flask-admin/blob/master/flask_admin/form/upload.py
"""

class BS3FileUploadFieldWidget(object):
    
    empty_template = ('<div class="input-group">'
                    '<span class="input-group-addon"><span class="glyphicon glyphicon-upload"></span>'
                    '</span>'
                    '<input class="form-control" %(file)s/>'
        '</div>'
        )
    
    data_template = ('<div>'
                     ' <input %(text)s>'
                     ' <input type="checkbox" name="%(marker)s">Delete</input>'
                     '</div>'
                     '<div class="input-group">'
                    '<span class="input-group-addon"><span class="glyphicon glyphicon-upload"></span>'
                    '</span>'
                    '<input class="form-control" %(file)s/>'
        '</div>'
        )

    def __call__(self, field, **kwargs):
        kwargs.setdefault('id', field.id)
        kwargs.setdefault('name', field.name)
        
        args = {
            'file': html_params(type='file',
                                **kwargs),
            'marker': '_%s-delete' % field.name
        }
        
        template = self.data_template if field.data else self.empty_template

        return HTMLString(template % {
            'text': html_params(type='text',
                                value=field.data),
            'file': html_params(type='file',
                                **kwargs),
            'marker': '_%s-delete' % field.name
        })
        

class BS3ImageUploadFieldWidget(object):
    
    empty_template = ('<div class="input-group">'
                    '<span class="input-group-addon"><span class="glyphicon glyphicon-upload"></span>'
                    '</span>'
                    '<input class="form-control" %(file)s/>'
        '</div>'
        )
    
    data_template = ('<div class="thumbnail">'
                     ' <img %(image)s>'
                     ' <input type="checkbox" name="%(marker)s">Delete</input>'
                     '</div>'
                     '<div class="input-group">'
                    '<span class="input-group-addon"><span class="glyphicon glyphicon-upload"></span>'
                    '</span>'
                    '<input class="form-control" %(file)s/>'
                    '</div>'
        )

    def __call__(self, field, **kwargs):
        kwargs.setdefault('id', field.id)
        kwargs.setdefault('name', field.name)
        
        args = {
            'file': html_params(type='file',
                                **kwargs),
            'marker': '_%s-delete' % field.name
        }
        
        if field.data:
            url = self.get_url(field)
            args['image'] = html_params(src=url)
            template = self.data_template
            
        else: 
            template = self.empty_template

        return HTMLString(template % args)

    def get_url(self, field):
        im = ImageManager()
        return im.get_url(field.data)


# Fields
class FileUploadField(fields.TextField):
    """
        Customizable file-upload field.

        Saves file to configured path, handles updates and deletions. Inherits from `TextField`,
        resulting filename will be stored as string.
    """
    widget = BS3FileUploadFieldWidget()

    def __init__(self, label=None, validators=None,
                 filemanager = None,
                 **kwargs):
        """
            Constructor.

            :param label:
                Display label
            :param validators:
                Validators
        """
        
        self.filemanager = filemanager or FileManager()
        self._should_delete = False

        super(FileUploadField, self).__init__(label, validators, **kwargs)

    def pre_validate(self, form):
        if (self.data and
                isinstance(self.data, FileStorage) and
                not self.filemanager.is_file_allowed(self.data.filename)):
            raise ValidationError(gettext('Invalid file extension'))

    def process(self, formdata, data=_unset_value):
        if formdata:
            marker = '_%s-delete' % self.name
            if marker in formdata:
                self._should_delete = True
        return super(FileUploadField, self).process(formdata, data)

    def populate_obj(self, obj, name):
        field = getattr(obj, name, None)
        if field:
            # If field should be deleted, clean it up
            if self._should_delete:
                self.filemanager.delete_file(field)
                setattr(obj, name, None)
                return

        if self.data and isinstance(self.data, FileStorage):
            if field:
                self.filemanager.delete_file(field)

            filename = self.filemanager.generate_name(obj, self.data)
            filename = self.filemanager.save_file(self.data, filename)

            setattr(obj, name, filename)

    

class ImageUploadField(fields.TextField):
    """
        Image upload field.
    """
    widget = BS3ImageUploadFieldWidget()

    def __init__(self, label=None, validators=None,
                 imagemanager = None,
                 **kwargs):
        
        self.imagemanager = imagemanager or ImageManager()
        self._should_delete = False
        super(ImageUploadField, self).__init__(label, validators, **kwargs)


    def pre_validate(self, form):
        if (self.data and
                isinstance(self.data, FileStorage) and
                not self.imagemanager.is_file_allowed(self.data.filename)):
            raise ValidationError(gettext('Invalid file extension'))

    def process(self, formdata, data=_unset_value):
        if formdata:
            marker = '_%s-delete' % self.name
            if marker in formdata:
                self._should_delete = True
        return super(ImageUploadField, self).process(formdata, data)


    def populate_obj(self, obj, name):
        field = getattr(obj, name, None)
        if field:
            # If field should be deleted, clean it up
            if self._should_delete:
                self.imagemanager.delete_file(field)
                setattr(obj, name, None)
                return

        if self.data and isinstance(self.data, FileStorage):
            if field:
                self.imagemanager.delete_file(field)

            filename = self.imagemanager.generate_name(obj, self.data)
            filename = self.imagemanager.save_file(self.data, filename)

            setattr(obj, name, filename)
    



import os
import sys
from setuptools import setup, find_packages

def fpath(name):
    return os.path.join(os.path.dirname(__file__), name)

def read(fname):
    return open(fpath(fname)).read()


def desc():
    return read('README.rst')

setup(
    name='Flask-AppBuilder',
    version='0.1.38',
    url='https://github.com/dpgaspar/flask-appbuilder/',
    license='BSD',
    author='Daniel Vaz Gaspar',
    author_email='danielvazgaspar@gmail.com',
    description='Simple and rapid Application builder, includes detailed security, auto form generation, google charts and much more.',
    long_description=desc(),
    packages=find_packages(),
    package_data={'': ['LICENSE']},
    scripts=['bin/init_app.py'],
    include_package_data=True,
    zip_safe=False,
    platforms='any',
    install_requires=[
        'Flask>=0.10',
        'Flask-Babel==0.8',
        'Flask-Login>=0.1.2',
        'Flask-OpenID>=1.1.0',
        'Flask-SQLAlchemy==0.16',
        'Flask-WTF==0.8.3',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ],
    test_suite=''
)



from flask.ext.wtf import Form, fields, widgets, TextField, BooleanField, TextAreaField,IntegerField, DateField,SelectFieldBase, SelectField, QuerySelectField, QuerySelectMultipleField, HiddenField
from flask.ext.wtf import Required, Length, validators, EqualTo
from wtforms.widgets import HTMLString, html_params
import sqlalchemy as sa
from upload import *


class GeneralModelConverter(object):

    def __init__(self, datamodel):
        self.datamodel = datamodel

    def _get_validators(self, colname, validators_columns):
        if colname in validators_columns:
            return validators_columns[colname]
        else:
            return []

    def _get_description(self, colname, description_columns):
        if colname in description_columns:
            return description_columns[colname]
        else:
            return ""

    def _get_label(self, colname, label_columns):
        if colname in label_columns:
            return label_columns[colname]
        else:
            return ""

    def _convert_many_to_one(self, prop, label, description, lst_validators, form_props):
        rel_model = self.datamodel.get_model_relation(prop)
        form_props[self.datamodel.get_property_col(prop)] = QuerySelectField(label,
                                                description=description,
                                                query_factory = lambda: self.datamodel.session.query(rel_model),
                                                allow_blank = True,
                                                widget=Select2Widget())
        return form_props

    def _convert_many_to_many(self, prop, label, description, lst_validators, form_props):
        rel_model = self.datamodel.get_model_relation(prop)
        form_props[self.datamodel.get_property_col(prop)] = QuerySelectMultipleField(label,
                description=description,
                query_factory = lambda: self.datamodel.session.query(rel_model),
                allow_blank=True,
                widget=Select2ManyWidget())
        return form_props

    def _convert_field(self, col, label, description, lst_validators, form_props):
        if not col.nullable:
            lst_validators.append(validators.Required())
        else:
            lst_validators.append(validators.Optional())
        if self.datamodel.is_image(col.name):
            form_props[col.name] = ImageUploadField(label,
                                    description=description,
                                    validators=lst_validators,
                                    widget=BS3ImageUploadFieldWidget())
        elif self.datamodel.is_file(col.name):
            form_props[col.name] = FileUploadField(label,
                                    description=description,
                                    validators=lst_validators,
                                    widget=BS3FileUploadFieldWidget())
        elif self.datamodel.is_text(col.name):
            form_props[col.name] = TextAreaField(label,
                                    description=description,
                                    validators=lst_validators,
                                    widget=BS3TextAreaFieldWidget())
        elif self.datamodel.is_string(col.name):
            form_props[col.name] = TextField(label,
                                    description=description,
                                    validators=lst_validators,
                                    widget=BS3TextFieldWidget())
        elif self.datamodel.is_integer(col.name):
            form_props[col.name] = IntegerField(label,
                                    description=description,
                                    validators=lst_validators,
                                    widget=BS3TextFieldWidget())
        elif self.datamodel.is_boolean(col.name):
            form_props[col.name] = BooleanField(label,
                                    description=description,
                                    validators=lst_validators)
        elif self.datamodel.is_date(col.name):
            form_props[col.name] = DateField(label,
                                                    description=description,
                                                    validators=lst_validators,
                                                    widget=DatePickerWidget())
        elif self.datamodel.is_datetime(col.name):
            form_props[col.name] = DateField(label,
                                    description=description,
                                    validators=lst_validators,
                                    widget=DateTimePickerWidget())
        else:
            print col.name + 'Column Type not supported!'
        return form_props

    def _convert_prop(self, prop, label, description, lst_validators, form_props):
        if self.datamodel.is_relation(prop):
            if self.datamodel.is_relation_many_to_one(prop):
                return self._convert_many_to_one(prop, label, description, lst_validators, form_props)
            if self.datamodel.is_relation_many_to_many(prop):
                return self._convert_many_to_many(prop, label, description, lst_validators, form_props)
        else:
            col = self.datamodel.get_property_first_col(prop)
            if not (self.datamodel.is_pk(col) or self.datamodel.is_fk(col)):
                return self._convert_field(col, label, description, lst_validators, form_props)


    def create_form(self, label_columns = {}, description_columns = {} ,validators_columns = {}, inc_columns = []):
        form_props = {}
        for col in inc_columns:
            prop = self.datamodel.get_col_property(col)
            self._convert_prop(prop, self._get_label(col, label_columns),
            self._get_description(col, description_columns),
            self._get_validators(col, validators_columns), form_props)
        return type('DynamicForm', (DynamicForm,), form_props)


class DynamicForm(Form):

    @classmethod
    def refresh(self, obj=None):
        form = self(obj=obj)
        return form

    @classmethod
    def debug(self):
        print self.__class__.__name__
        for item in self():
            print item.name

#------------------------------------------
#         WIDGETS
#------------------------------------------
class DatePickerWidget(object):
    """
    Date Time picker from Eonasdan GitHub

    """
    data_template = ('<div class="input-group date" id="datepicker">'
                    '<span class="input-group-addon"><span class="glyphicon glyphicon-calendar"></span>'
                    '</span>'
                    '<input class="form-control" data-format="yyyy-MM-dd" %(text)s/>'
                    '</div>'
                    )

    def __call__(self, field, **kwargs):
        kwargs.setdefault('id', field.id)
        kwargs.setdefault('name', field.name)
        template = self.data_template

        return HTMLString(template % {'text': html_params(type='text',
                                      value=field.data,
                                      **kwargs)
                                      })


class DateTimePickerWidget(object):
    """
    Date Time picker from Eonasdan GitHub

    """
    data_template = ('<div class="input-group date" id="datetimepicker">'
                    '<span class="input-group-addon"><span class="glyphicon glyphicon-calendar"></span>'
                    '</span>'
                    '<input class="form-control" data-format="yyyy-MM-dd hh:mm:ss" %(text)s/>'
        '</div>'
        )

    def __call__(self, field, **kwargs):
        kwargs.setdefault('id', field.id)
        kwargs.setdefault('name', field.name)
        template = self.data_template

        return HTMLString(template % {'text': html_params(type='text',
                                        value=field.data,
                                        **kwargs)
                                })




class BS3TextFieldWidget(widgets.TextInput):
    def __call__(self, field, **kwargs):
        kwargs['class'] = u'form-control'
        if field.label:
            kwargs['placeholder'] = field.label.text
        return super(BS3TextFieldWidget, self).__call__(field, **kwargs)

class BS3TextAreaFieldWidget(widgets.TextArea):
    def __call__(self, field, **kwargs):
        kwargs['class'] = u'form-control'
        kwargs['rows'] = 3
        if field.label:
            kwargs['placeholder'] = field.label.text
        return super(BS3TextAreaFieldWidget, self).__call__(field, **kwargs)

class BS3PasswordFieldWidget(widgets.PasswordInput):
    def __call__(self, field, **kwargs):
        kwargs['class'] = u'form-control'
        if field.label:
            kwargs['placeholder'] = field.label.text
        return super(BS3PasswordFieldWidget, self).__call__(field, **kwargs)


class Select2Widget(widgets.Select):
    def __call__(self, field, **kwargs):
        kwargs['class'] = u'my_select2'
        kwargs['style'] = u'width:250px'
        kwargs['data-placeholder'] = u'Select Value'
        return super(Select2Widget, self).__call__(field, **kwargs)

class Select2ManyWidget(widgets.Select):
    def __call__(self, field, **kwargs):
        kwargs['class'] = u'my_select2'
        kwargs['style'] = u'width:250px'
        kwargs['data-placeholder'] = u'Select Value'
        kwargs['multiple'] = u'true'
        return super(Select2ManyWidget, self).__call__(field, **kwargs)



'''
Created on Oct 12, 2013

@author: Daniel Gaspar
'''

from flask.globals import _request_ctx_stack


class RenderTemplateWidget(object):

    template = 'appbuilder/general/widgets/render.html'

    def __call__(self, **kwargs):
        ctx = _request_ctx_stack.top
        jinja_env = ctx.app.jinja_env

        template = jinja_env.get_template(self.template)
        return template.render(kwargs)


class FormWidget(RenderTemplateWidget):

    template = 'appbuilder/general/widgets/form.html'

    route_base = ''
    form = None
    exclude_cols = []
    fieldsets = []

    def __init__(self, route_base='', form=None, exclude_cols=[], fieldsets = []):
        self.route_base = route_base
        self.form = form
        self.exclude_cols = exclude_cols
        self.fieldsets = fieldsets


    def __call__(self, **kwargs):
        kwargs['route_base'] = self.route_base
        kwargs['form'] = self.form
        kwargs['exclude_cols'] = self.exclude_cols
        kwargs['fieldsets'] = self.fieldsets
        return super(FormWidget, self).__call__(**kwargs)

class SearchWidget(FormWidget):
    template = 'appbuilder/general/widgets/search.html'
    def __init__(self, **kwargs):
        return super(SearchWidget, self).__init__(**kwargs)

class ShowWidget(RenderTemplateWidget):

    template = 'appbuilder/general/widgets/show.html'

    route_base = ''
    label_columns = []
    include_columns = []
    value_columns = []
    additional_links = []
    fieldsets = []

    def __init__(self, route_base = '',
                label_columns = [],
                include_columns = [],
                value_columns = [],
                additional_links = [],
                fieldsets = []):
        self.route_base = route_base
        self.label_columns = label_columns
        self.include_columns = include_columns
        self.value_columns = value_columns
        self.additional_links = additional_links
        self.fieldsets = fieldsets

    def __call__(self, **kwargs):
        kwargs['route_base'] = self.route_base
        kwargs['label_columns'] = self.label_columns
        kwargs['include_columns'] = self.include_columns
        kwargs['value_columns'] = self.value_columns
        kwargs['additional_links'] = self.additional_links
        kwargs['fieldsets'] = self.fieldsets
        return super(ShowWidget, self).__call__(**kwargs)


class ListWidget(RenderTemplateWidget):

    template = 'appbuilder/general/widgets/list.html'

    route_base = ''
    label_columns = []
    include_columns = []
    value_columns = []
    order_columns = []
    pks = []
    filters = {}
    generalview_name = ''

    def __init__(self, route_base = '',
                 label_columns = [],
                 include_columns = [],
                 value_columns = [],
                 order_columns = [],
                 pks = [],
                 filters = {},
                 generalview_name = ''):
        self.route_base = route_base
        self.label_columns = label_columns
        self.include_columns = include_columns
        self.value_columns = value_columns
        self.order_columns = order_columns
        self.pks = pks
        self.filters = filters
        self.generalview_name = generalview_name

    def __call__(self, **kwargs):
        kwargs['route_base'] = self.route_base
        kwargs['label_columns'] = self.label_columns
        kwargs['include_columns'] = self.include_columns
        kwargs['value_columns'] = self.value_columns
        kwargs['order_columns'] = self.order_columns
        kwargs['pks'] = self.pks
        kwargs['filters'] = self.filters
        kwargs['generalview_name'] = self.generalview_name
        return super(ListWidget, self).__call__(**kwargs)
        
class ListThumbnail(ListWidget):
    template = 'appbuilder/general/widgets/list_thumbnail.html'



from flask import g, request, current_app
from flask.ext.login import current_user
from flask import flash, redirect,url_for
from models import is_item_public

def has_access(f):
    """
        Use this decorator to allow access only to security 
        defined permissions
    """
    def wrap(self, *args, **kwargs):
        if current_user.is_authenticated():
            if g.user.has_permission_on_view("can_" + f.__name__, self.__class__.__name__):
                return f(self, *args, **kwargs)
            else:
                flash("Access is Denied %s %s" % (f.__name__, self.__class__.__name__),"danger")
        else:
            if is_item_public("can_" + f.__name__, self.__class__.__name__):
                return f(self, *args, **kwargs)
            else:
                flash("Access is Denied %s %s" % (f.__name__, self.__class__.__name__),"danger")
        return redirect(url_for("authview.login"))
    return wrap

